@@ -34,7 +34,6 @@
 
 #define ALSA_PCM_NEW_HW_PARAMS_API
 #include <alsa/asoundlib.h>
-#include <pthread.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
@@ -49,11 +48,6 @@
 
 #define BUFFER_LENGTH 512
 
-/* SCHED_FIFO priorities for ALSA threads (see pthread_attr_setschedparam) */
-#define ALSA_PCM_SCHED_PRIORITY 90
-#define ALSA_RAWMIDI_SCHED_PRIORITY 90
-#define ALSA_SEQ_SCHED_PRIORITY 90
-
 /** fluid_alsa_audio_driver_t
  *
  * This structure should not be accessed directly. Use audio port
@@ -65,7 +59,7 @@ typedef struct {
   fluid_audio_func_t callback;
   void* data;
   int buffer_size;
-  pthread_t thread;
+  fluid_thread_t *thread;
   int cont;
 } fluid_alsa_audio_driver_t;
 
@@ -77,15 +71,15 @@ fluid_audio_driver_t* new_fluid_alsa_audio_driver2(fluid_settings_t* settings,
 
 int delete_fluid_alsa_audio_driver(fluid_audio_driver_t* p);
 void fluid_alsa_audio_driver_settings(fluid_settings_t* settings);
-static void* fluid_alsa_audio_run_float(void* d);
-static void* fluid_alsa_audio_run_s16(void* d);
+static void fluid_alsa_audio_run_float(void* d);
+static void fluid_alsa_audio_run_s16(void* d);
 
 
 struct fluid_alsa_formats_t {
   char* name;
   snd_pcm_format_t format;
   snd_pcm_access_t access;
-  void* (*run)(void* d);
+  fluid_thread_func_t run;
 };
 
 struct fluid_alsa_formats_t fluid_alsa_formats[] = {
@@ -111,7 +105,7 @@ typedef struct {
   snd_rawmidi_t *rawmidi_in;
   struct pollfd *pfd;
   int npfd;
-  pthread_t thread;
+  fluid_thread_t *thread;
   int status;
   unsigned char buffer[BUFFER_LENGTH];
   fluid_midi_parser_t* parser;
@@ -123,7 +117,7 @@ fluid_midi_driver_t* new_fluid_alsa_rawmidi_driver(fluid_settings_t* settings,
 						   void* event_handler_data);
 
 int delete_fluid_alsa_rawmidi_driver(fluid_midi_driver_t* p);
-static void* fluid_alsa_midi_run(void* d);
+static void fluid_alsa_midi_run(void* d);
 
 
 /*
@@ -135,7 +129,7 @@ typedef struct {
   snd_seq_t *seq_handle;
   struct pollfd *pfd;
   int npfd;
-  pthread_t thread;
+  fluid_thread_t *thread;
   int status;
   int port_count;
 } fluid_alsa_seq_driver_t;
@@ -144,7 +138,7 @@ fluid_midi_driver_t* new_fluid_alsa_seq_driver(fluid_settings_t* settings,
 					     handle_midi_event_func_t handler,
 					     void* data);
 int delete_fluid_alsa_seq_driver(fluid_midi_driver_t* p);
-static void* fluid_alsa_seq_run(void* d);
+static void fluid_alsa_seq_run(void* d);
 
 /**************************************************************
  *
@@ -172,10 +166,8 @@ new_fluid_alsa_audio_driver2(fluid_settings_t* settings,
   fluid_alsa_audio_driver_t* dev;
   double sample_rate;
   int periods, period_size;
-  char* device;
-  pthread_attr_t attr;
-  int sched = SCHED_FIFO;
-  struct sched_param priority;
+  char* device = NULL;
+  int realtime_prio = 0;
   int i, err, dir = 0;
   snd_pcm_hw_params_t* hwparams;
   snd_pcm_sw_params_t* swparams = NULL;
@@ -192,7 +184,8 @@ new_fluid_alsa_audio_driver2(fluid_settings_t* settings,
   fluid_settings_getint(settings, "audio.periods", &periods);
   fluid_settings_getint(settings, "audio.period-size", &period_size);
   fluid_settings_getnum(settings, "synth.sample-rate", &sample_rate);
-  fluid_settings_getstr(settings, "audio.alsa.device", &device);
+  fluid_settings_dupstr(settings, "audio.alsa.device", &device);   /* ++ dup device name */
+  fluid_settings_getint (settings, "audio.realtime-prio", &realtime_prio);
 
   dev->data = data;
   dev->callback = func;
@@ -200,12 +193,15 @@ new_fluid_alsa_audio_driver2(fluid_settings_t* settings,
   dev->buffer_size = period_size;
 
   /* Open the PCM device */
-  if ((err = snd_pcm_open(&dev->pcm, device, SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK)) != 0) {
+  if ((err = snd_pcm_open(&dev->pcm, device ? device : "default",
+                          SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK)) != 0) {
     if (err == -EBUSY) {
-      FLUID_LOG(FLUID_ERR, "The \"%s\" audio device is used by another application", device);
+      FLUID_LOG(FLUID_ERR, "The \"%s\" audio device is used by another application",
+                device ? device : "default");
       goto error_recovery;
     } else {
-      FLUID_LOG(FLUID_ERR, "Failed to open the \"%s\" audio device", device);
+      FLUID_LOG(FLUID_ERR, "Failed to open the \"%s\" audio device",
+                device ? device : "default");
       goto error_recovery;
     }
   }
@@ -247,7 +243,7 @@ new_fluid_alsa_audio_driver2(fluid_settings_t* settings,
       /* There's currently no way to change the sampling rate of the
 	 synthesizer after it's been created. */
       FLUID_LOG(FLUID_WARN, "Requested sample rate of %d, got %d instead, "
-		"synthesizer likely out of tune!", sample_rate, tmp);
+		"synthesizer likely out of tune!", (unsigned int) sample_rate, tmp);
     }
 
     uframes = period_size;
@@ -285,8 +281,6 @@ new_fluid_alsa_audio_driver2(fluid_settings_t* settings,
     goto error_recovery;
   }
 
-  FLUID_LOG(FLUID_INFO, "ALSA driver: Using format %s", fluid_alsa_formats[i].name);
-
   /* Set the software params */
   snd_pcm_sw_params_current(dev->pcm, swparams);
 
@@ -294,22 +288,6 @@ new_fluid_alsa_audio_driver2(fluid_settings_t* settings,
     FLUID_LOG(FLUID_ERR, "Failed to set start threshold.");
   }
 
-  /* FIXME - Any of these calls important?  One of them was causing massive
-     ALSA CPU consumption! */
-
-//  if (snd_pcm_sw_params_set_stop_threshold(dev->pcm, swparams, ~0u) != 0) {
-//    FLUID_LOG(FLUID_ERR, "Cannot turn off stop threshold.");
-//  }
-
-//  if (snd_pcm_sw_params_set_silence_threshold(dev->pcm, swparams, 0) != 0) {
-//    FLUID_LOG(FLUID_ERR, "Cannot set 0 silence threshold.");
-//  }
-
-//  if (snd_pcm_sw_params_set_silence_size(dev->pcm, swparams, 0) != 0) {
-//    FLUID_LOG(FLUID_ERR, "Cannot set 0 silence size.");
-//  }
-
-//  if (snd_pcm_sw_params_set_avail_min(dev->pcm, swparams, period_size / 2) != 0) {
   if (snd_pcm_sw_params_set_avail_min(dev->pcm, swparams, period_size) != 0) {
     FLUID_LOG(FLUID_ERR, "Software setup for minimum available frames failed.");
   }
@@ -320,51 +298,17 @@ new_fluid_alsa_audio_driver2(fluid_settings_t* settings,
 
 
   /* Create the audio thread */
+  dev->thread = new_fluid_thread (fluid_alsa_formats[i].run, dev, realtime_prio, FALSE);
 
-  if (pthread_attr_init(&attr)) {
-    FLUID_LOG(FLUID_ERR, "Couldn't initialize audio thread attributes");
+  if (!dev->thread)
     goto error_recovery;
-  }
 
-  /* The pthread_create man page explains that
-     pthread_attr_setschedpolicy returns an error if the user is not
-     permitted the set SCHED_FIFO. It seems however that no error is
-     returned but pthread_create fails instead. That's why I try to
-     create the thread twice in a while loop. */
-  while (1) {
-    err = pthread_attr_setschedpolicy(&attr, sched);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the audio output");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_ERR, "Couldn't set scheduling policy.");
-	goto error_recovery;
-      }
-    }
-
-    /* SCHED_FIFO will not be active without setting the priority */
-    priority.sched_priority = (sched == SCHED_FIFO) ? ALSA_PCM_SCHED_PRIORITY : 0;
-    pthread_attr_setschedparam(&attr, &priority);
-
-    err = pthread_create(&dev->thread, &attr, fluid_alsa_formats[i].run, (void*) dev);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the audio output");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_PANIC, "Couldn't create the audio thread.");
-	goto error_recovery;
-      }
-    }
-    break;
-  }
+  if (device) FLUID_FREE (device);      /* -- free device name */
 
   return (fluid_audio_driver_t*) dev;
 
  error_recovery:
+  if (device) FLUID_FREE (device);      /* -- free device name */
   delete_fluid_alsa_audio_driver((fluid_audio_driver_t*) dev);
   return NULL;
 }
@@ -379,12 +323,8 @@ int delete_fluid_alsa_audio_driver(fluid_audio_driver_t* p)
 
   dev->cont = 0;
 
-  if (dev->thread) {
-    if (pthread_join(dev->thread, NULL)) {
-      FLUID_LOG(FLUID_ERR, "Failed to join the audio thread");
-      return FLUID_FAILED;
-    }
-  }
+  if (dev->thread)
+    fluid_thread_join (dev->thread);
 
   if (dev->pcm) {
     snd_pcm_state_t state = snd_pcm_state(dev->pcm);
@@ -431,7 +371,7 @@ static int fluid_alsa_handle_write_error (snd_pcm_t *pcm, int errval)
   return FLUID_OK;
 }
 
-static void* fluid_alsa_audio_run_float(void* d)
+static void fluid_alsa_audio_run_float (void *d)
 {
   fluid_alsa_audio_driver_t* dev = (fluid_alsa_audio_driver_t*) d;
   fluid_synth_t *synth = (fluid_synth_t *)(dev->data);
@@ -447,7 +387,7 @@ static void* fluid_alsa_audio_run_float(void* d)
 
   if ((left == NULL) || (right == NULL)) {
     FLUID_LOG(FLUID_ERR, "Out of memory.");
-    return NULL;
+    return;
   }
 
   if (snd_pcm_nonblock(dev->pcm, 0) != 0) { /* double negation */
@@ -509,19 +449,16 @@ static void* fluid_alsa_audio_run_float(void* d)
 
   FLUID_FREE(left);
   FLUID_FREE(right);
-
-  return NULL;
 }
 
-static void* fluid_alsa_audio_run_s16(void* d)
+static void fluid_alsa_audio_run_s16 (void *d)
 {
   fluid_alsa_audio_driver_t* dev = (fluid_alsa_audio_driver_t*) d;
   float* left;
   float* right;
   short* buf;
   float* handle[2];
-  int i, k, n, buffer_size, offset;
-  float s;
+  int n, buffer_size, offset;
 
   buffer_size = dev->buffer_size;
 
@@ -531,7 +468,7 @@ static void* fluid_alsa_audio_run_s16(void* d)
 
   if ((left == NULL) || (right == NULL) || (buf == NULL)) {
     FLUID_LOG(FLUID_ERR, "Out of memory.");
-    return NULL;
+    return;
   }
 
   handle[0] = left;
@@ -602,8 +539,6 @@ static void* fluid_alsa_audio_run_s16(void* d)
   FLUID_FREE(left);
   FLUID_FREE(right);
   FLUID_FREE(buf);
-
-  return NULL;
 }
 
 
@@ -629,9 +564,7 @@ new_fluid_alsa_rawmidi_driver(fluid_settings_t* settings,
 {
   int i, err;
   fluid_alsa_rawmidi_driver_t* dev;
-  pthread_attr_t attr;
-  int sched = SCHED_FIFO;
-  struct sched_param priority;
+  int realtime_prio = 0;
   int count;
   struct pollfd *pfd = NULL;
   char* device = NULL;
@@ -660,18 +593,20 @@ new_fluid_alsa_rawmidi_driver(fluid_settings_t* settings,
     goto error_recovery;
   }
 
+  fluid_settings_getint (settings, "midi.realtime-prio", &realtime_prio);
+
   /* get the device name. if none is specified, use the default device. */
-  fluid_settings_getstr(settings, "midi.alsa.device", &device);
-  if (device == NULL) {
-    device = "default";
-  }
+  fluid_settings_dupstr(settings, "midi.alsa.device", &device);         /* ++ alloc device name */
 
   /* open the hardware device. only use midi in. */
-  if ((err = snd_rawmidi_open(&dev->rawmidi_in, NULL, device, SND_RAWMIDI_NONBLOCK)) < 0) {
+  if ((err = snd_rawmidi_open(&dev->rawmidi_in, NULL, device ? device : "default",
+                              SND_RAWMIDI_NONBLOCK)) < 0) {
     FLUID_LOG(FLUID_ERR, "Error opening ALSA raw MIDI port");
     goto error_recovery;
   }
 
+  snd_rawmidi_nonblock(dev->rawmidi_in, 1);
+
   /* get # of MIDI file descriptors */
   count = snd_rawmidi_poll_descriptors_count(dev->rawmidi_in);
   if (count > 0) {		/* make sure there are some */
@@ -694,61 +629,18 @@ new_fluid_alsa_rawmidi_driver(fluid_settings_t* settings,
 
   dev->status = FLUID_MIDI_READY;
 
-  /* create the midi thread */
-  if (pthread_attr_init(&attr)) {
-    FLUID_LOG(FLUID_ERR, "Couldn't initialize midi thread attributes");
+  /* create the MIDI thread */
+  dev->thread = new_fluid_thread (fluid_alsa_midi_run, dev, realtime_prio, FALSE);
+
+  if (!dev->thread)
     goto error_recovery;
-  }
 
-  /* Was: "use fifo scheduling. if it fails, use default scheduling." */
-  /* Now normal scheduling is used by default for the MIDI thread. The reason is,
-   * that fluidsynth works better with low latencies under heavy load, if only the
-   * audio thread is prioritized.
-   * With MIDI at ordinary priority, that could result in individual notes being played
-   * a bit late. On the other hand, if the audio thread is delayed, an audible dropout
-   * is the result.
-   * To reproduce this: Edirol UA-1 USB-MIDI interface, four buffers
-   * with 45 samples each (roughly 4 ms latency), ravewave soundfont. -MN
-   */
+  if (device) FLUID_FREE (device);      /* -- free device name */
 
-  /* Not so sure anymore. We're losing MIDI data, if we can't keep up with
-   * the speed it is generated. */
-/*  FLUID_LOG(FLUID_WARN, "Note: High-priority scheduling for the MIDI thread was intentionally disabled.");
-    sched=SCHED_OTHER;*/
-
-  while (1) {
-    err = pthread_attr_setschedpolicy(&attr, sched);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the MIDI input");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_ERR, "Couldn't set scheduling policy.");
-	goto error_recovery;
-      }
-    }
-
-    /* SCHED_FIFO will not be active without setting the priority */
-    priority.sched_priority = (sched == SCHED_FIFO) ? ALSA_RAWMIDI_SCHED_PRIORITY : 0;
-    pthread_attr_setschedparam (&attr, &priority);
-
-    err = pthread_create(&dev->thread, &attr, fluid_alsa_midi_run, (void*) dev);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the MIDI input");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_PANIC, "Couldn't create the midi thread.");
-	goto error_recovery;
-      }
-    }
-    break;
-  }
   return (fluid_midi_driver_t*) dev;
 
  error_recovery:
+  if (device) FLUID_FREE (device);      /* -- free device name */
   delete_fluid_alsa_rawmidi_driver((fluid_midi_driver_t*) dev);
   return NULL;
 
@@ -767,19 +659,12 @@ delete_fluid_alsa_rawmidi_driver(fluid_midi_driver_t* p)
     return FLUID_OK;
   }
 
+  /* cancel the thread and wait for it before cleaning up */
   dev->status = FLUID_MIDI_DONE;
 
-  /* cancel the thread and wait for it before cleaning up */
-  if (dev->thread) {
-    if (pthread_cancel(dev->thread)) {
-      FLUID_LOG(FLUID_ERR, "Failed to cancel the midi thread");
-      return FLUID_FAILED;
-    }
-    if (pthread_join(dev->thread, NULL)) {
-      FLUID_LOG(FLUID_ERR, "Failed to join the midi thread");
-      return FLUID_FAILED;
-    }
-  }
+  if (dev->thread)
+    fluid_thread_join (dev->thread);
+
   if (dev->rawmidi_in) {
     snd_rawmidi_close(dev->rawmidi_in);
   }
@@ -793,22 +678,12 @@ delete_fluid_alsa_rawmidi_driver(fluid_midi_driver_t* p)
 /*
  * fluid_alsa_midi_run
  */
-void*
+void
 fluid_alsa_midi_run(void* d)
 {
-  int n, i;
   fluid_midi_event_t* evt;
   fluid_alsa_rawmidi_driver_t* dev = (fluid_alsa_rawmidi_driver_t*) d;
-
-  /* make sure the other threads can cancel this thread any time */
-  if (pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL)) {
-    FLUID_LOG(FLUID_ERR, "Failed to set the cancel state of the midi thread");
-    pthread_exit(NULL);
-  }
-  if (pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL)) {
-    FLUID_LOG(FLUID_ERR, "Failed to set the cancel state of the midi thread");
-    pthread_exit(NULL);
-  }
+  int n, i;
 
   /* go into a loop until someone tells us to stop */
   dev->status = FLUID_MIDI_LISTENING;
@@ -836,7 +711,6 @@ fluid_alsa_midi_run(void* d)
       }
     }
   }
-  pthread_exit(NULL);
 }
 
 /**************************************************************
@@ -892,17 +766,14 @@ new_fluid_alsa_seq_driver(fluid_settings_t* settings,
 {
   int i, err;
   fluid_alsa_seq_driver_t* dev;
-  pthread_attr_t attr;
-  int sched = SCHED_FIFO;
-  struct sched_param priority;
+  int realtime_prio = 0;
   int count;
   struct pollfd *pfd = NULL;
-  char * device = NULL;
-  char * id;
-  char * portname;
+  char *device = NULL;
+  char *id = NULL;
+  char *portname = NULL;
   char full_id[64];
   char full_name[64];
-  char id_pid[16];
   snd_seq_port_info_t *port_info = NULL;
   int midi_channels;
 
@@ -922,31 +793,43 @@ new_fluid_alsa_seq_driver(fluid_settings_t* settings,
   dev->driver.data = data;
   dev->driver.handler = handler;
 
+  fluid_settings_getint (settings, "midi.realtime-prio", &realtime_prio);
 
   /* get the device name. if none is specified, use the default device. */
-  fluid_settings_getstr(settings, "midi.alsa_seq.device", &device);
-  if (device == NULL) {
-    device = "default";
-  }
+  if (fluid_settings_dupstr(settings, "midi.alsa_seq.device", &device) == 0)    /* ++ alloc device name */
+    goto error_recovery;
+
+  if (fluid_settings_dupstr(settings, "midi.alsa_seq.id", &id) == 0)    /* ++ alloc id string */
+    goto error_recovery;
 
-  fluid_settings_getstr(settings, "midi.alsa_seq.id", &id);
   if (id == NULL) {
-    sprintf(id_pid, "%d", getpid());
-    id = id_pid;
+    id = FLUID_MALLOC (32);
+    if (!id)
+    {
+      FLUID_LOG(FLUID_ERR, "Out of memory");
+      goto error_recovery;
+    }
+
+    sprintf(id, "%d", getpid());
   }
 
   /* get the midi portname */
-  fluid_settings_getstr(settings, "midi.portname", &portname);
-  if (!strcmp(portname, ""))
-      portname = NULL;
+  fluid_settings_dupstr(settings, "midi.portname", &portname);
+  if (portname && FLUID_STRLEN (portname) == 0)
+  {
+    FLUID_FREE (portname);      /* -- free port name */
+    portname = NULL;
+  }
 
   /* open the sequencer INPUT only */
-  err = snd_seq_open(&dev->seq_handle, device, SND_SEQ_OPEN_INPUT, 0);
+  err = snd_seq_open(&dev->seq_handle, device ? device : "default", SND_SEQ_OPEN_INPUT, 0);
   if (err < 0) {
     FLUID_LOG(FLUID_ERR, "Error opening ALSA sequencer");
     goto error_recovery;
   }
 
+  snd_seq_nonblock (dev->seq_handle, 1);
+
   /* get # of MIDI file descriptors */
   count = snd_seq_poll_descriptors_count(dev->seq_handle, POLLIN);
   if (count > 0) {		/* make sure there are some */
@@ -1020,46 +903,23 @@ new_fluid_alsa_seq_driver(fluid_settings_t* settings,
 
   dev->status = FLUID_MIDI_READY;
 
-  /* create the midi thread */
-  if (pthread_attr_init(&attr)) {
-    FLUID_LOG(FLUID_ERR, "Couldn't initialize midi thread attributes");
-    goto error_recovery;
-  }
-  /* use fifo scheduling. if it fails, use default scheduling. */
-  while (1) {
-    err = pthread_attr_setschedpolicy(&attr, sched);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the MIDI input");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_ERR, "Couldn't set scheduling policy.");
-	goto error_recovery;
-      }
-    }
+  /* create the MIDI thread */
+  dev->thread = new_fluid_thread (fluid_alsa_seq_run, dev, realtime_prio, FALSE);
 
-    /* SCHED_FIFO will not be active without setting the priority */
-    priority.sched_priority = (sched == SCHED_FIFO) ? ALSA_SEQ_SCHED_PRIORITY : 0;
-    pthread_attr_setschedparam (&attr, &priority);
+  if (portname) FLUID_FREE (portname);
+  if (id) FLUID_FREE (id);
+  if (device) FLUID_FREE (device);
 
-    err = pthread_create(&dev->thread, &attr, fluid_alsa_seq_run, (void*) dev);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the MIDI input");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_PANIC, "Couldn't create the midi thread.");
-	goto error_recovery;
-      }
-    }
-    break;
-  }
   return (fluid_midi_driver_t*) dev;
 
  error_recovery:
+
+  if (portname) FLUID_FREE (portname);
+  if (id) FLUID_FREE (id);
+  if (device) FLUID_FREE (device);
+
   delete_fluid_alsa_seq_driver((fluid_midi_driver_t*) dev);
+
   return NULL;
 }
 
@@ -1076,19 +936,12 @@ delete_fluid_alsa_seq_driver(fluid_midi_driver_t* p)
     return FLUID_OK;
   }
 
+  /* cancel the thread and wait for it before cleaning up */
   dev->status = FLUID_MIDI_DONE;
 
-  /* cancel the thread and wait for it before cleaning up */
-  if (dev->thread) {
-    if (pthread_cancel(dev->thread)) {
-      FLUID_LOG(FLUID_ERR, "Failed to cancel the midi thread");
-      return FLUID_FAILED;
-    }
-    if (pthread_join(dev->thread, NULL)) {
-      FLUID_LOG(FLUID_ERR, "Failed to join the midi thread");
-      return FLUID_FAILED;
-    }
-  }
+  if (dev->thread)
+    fluid_thread_join (dev->thread);
+
   if (dev->seq_handle) {
     snd_seq_close(dev->seq_handle);
   }
@@ -1102,24 +955,13 @@ delete_fluid_alsa_seq_driver(fluid_midi_driver_t* p)
 /*
  * fluid_alsa_seq_run
  */
-void*
+void
 fluid_alsa_seq_run(void* d)
 {
   int n, ev;
   snd_seq_event_t *seq_ev;
   fluid_midi_event_t evt;
   fluid_alsa_seq_driver_t* dev = (fluid_alsa_seq_driver_t*) d;
-  int channel;
-
-  /* make sure the other threads can cancel this thread any time */
-  if (pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL)) {
-    FLUID_LOG(FLUID_ERR, "Failed to set the cancel state of the midi thread");
-    pthread_exit(NULL);
-  }
-  if (pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL)) {
-    FLUID_LOG(FLUID_ERR, "Failed to set the cancel state of the midi thread");
-    pthread_exit(NULL);
-  }
 
   /* go into a loop until someone tells us to stop */
   dev->status = FLUID_MIDI_LISTENING;
@@ -1134,6 +976,8 @@ fluid_alsa_seq_run(void* d)
 	{
 	    ev = snd_seq_event_input(dev->seq_handle, &seq_ev);	/* read the events */
 
+            if (ev == -EAGAIN) break;
+
 	    /* Negative value indicates an error, ignore interrupted system call
 	     * (-EPERM) and input event buffer overrun (-ENOSPC) */
 	    if (ev < 0)
@@ -1189,6 +1033,12 @@ fluid_alsa_seq_run(void* d)
 	      evt.channel = seq_ev->dest.port * 16 + seq_ev->data.control.channel;
 	      evt.param1 = seq_ev->data.control.value;
 	      break;
+	    case SND_SEQ_EVENT_SYSEX:
+	      if (seq_ev->data.ext.len < 2) continue;
+
+	      fluid_midi_event_set_sysex (&evt, (char *)(seq_ev->data.ext.ptr) + 1,
+					  seq_ev->data.ext.len - 2, FALSE);
+	      break;
 	    default:
 	      continue;		/* unhandled event, next loop iteration */
 	    }
@@ -1199,7 +1049,6 @@ fluid_alsa_seq_run(void* d)
 	while (ev > 0);
     }	/* if poll() > 0 */
   }	/* while (dev->status == FLUID_MIDI_LISTENING) */
-  pthread_exit(NULL);
 }
 
 #endif /* #if ALSA_SUPPORT */
