@@ -67,8 +67,8 @@ void fluid_winmidi_midi_driver_settings(fluid_settings_t* settings);
 /* definitions for the MidiShare driver */
 #if MIDISHARE_SUPPORT
 fluid_midi_driver_t* new_fluid_midishare_midi_driver(fluid_settings_t* settings,
-						   void* event_handler_data,
-						   handle_midi_event_func_t handler);
+                                                     handle_midi_event_func_t handler,
+                                                     void* event_handler_data);
 int delete_fluid_midishare_midi_driver(fluid_midi_driver_t* p);
 #endif
 
@@ -145,6 +145,9 @@ void fluid_midi_driver_settings(fluid_settings_t* settings)
 {
   int i;
 
+  fluid_settings_register_int (settings, "midi.realtime-prio",
+                               FLUID_DEFAULT_MIDI_RT_PRIO, 0, 99, 0, NULL, NULL);
+
   /* Set the default driver */
 #if ALSA_SUPPORT
   fluid_settings_register_str(settings, "midi.driver", "alsa_seq", 0, NULL, NULL);
@@ -190,7 +193,6 @@ void fluid_midi_driver_settings(fluid_settings_t* settings)
   }
 }
 
-
 /**
  * Create a new MIDI driver instance.
  * @param settings Settings used to configure new MIDI driver.
@@ -201,8 +203,10 @@ void fluid_midi_driver_settings(fluid_settings_t* settings)
  */
 fluid_midi_driver_t* new_fluid_midi_driver(fluid_settings_t* settings, handle_midi_event_func_t handler, void* event_handler_data)
 {
-  int i;
   fluid_midi_driver_t* driver = NULL;
+  char *allnames;
+  int i;
+
   for (i = 0; fluid_midi_drivers[i].name != NULL; i++) {
     if (fluid_settings_str_equal(settings, "midi.driver", fluid_midi_drivers[i].name)) {
       FLUID_LOG(FLUID_DBG, "Using '%s' midi driver", fluid_midi_drivers[i].name);
@@ -214,7 +218,11 @@ fluid_midi_driver_t* new_fluid_midi_driver(fluid_settings_t* settings, handle_mi
     }
   }
 
-  FLUID_LOG(FLUID_ERR, "Couldn't find the requested midi driver");
+  allnames = fluid_settings_option_concat (settings, "midi.driver", NULL);
+  FLUID_LOG(FLUID_ERR, "Couldn't find the requested midi driver. Valid drivers are: %s.",
+            allnames ? allnames : "ERROR");
+  if (allnames) FLUID_FREE (allnames);
+
   return NULL;
 }
 
